import React, { Component } from 'react';
import { StyleSheet, Text, View } from 'react-native';
import PropTypes from 'prop-types';
import { TextInputMask } from 'react-native-masked-text';
import { parsePhoneNumberFromString } from 'libphonenumber-js';
import Flag from 'react-native-flags';
import {
  getMaskToLibPhoneNumber,
  findFirstCountryByCode,
  isCanada,
} from './country_telephone_data';

const ipstackUrl =
  'http://api.ipstack.com/89.22.47.182?access_key=a2403e5b1e542843a8b8621bd2437586';

class PhoneMaskInput extends Component {
  constructor(props) {
    super(props);

    fetch(ipstackUrl)
      .then(response => response.json())
      .then(responseJson => {
        this.updateMaskData(this.props.value, responseJson.country_code);
      })
      .catch(error => {});

    this.state = {
      inputValue: this.props.value,
      country: '',
      countryCode: '',
      mask: '+999999999999',
      defaultMask: '+999999999999',
    };
  }

  updateMaskData = function(inputNumber, visitorCountry) {
    const { countryCode, defaultMask } = this.state;

    let phoneInfo = parsePhoneNumberFromString(inputNumber);
    if (phoneInfo && !phoneInfo.country) {
      switch (phoneInfo.countryCallingCode) {
        case '44': {
          phoneInfo.country = 'GB';
          break;
        }
        case '1': {
          if (inputNumber.match(/\d/g).length > 3)
            phoneInfo.country = isCanada(inputNumber) ? 'CA' : 'US';
          break;
        }
      }
    } else if (visitorCountry && !inputNumber) phoneInfo = { country: visitorCountry };
    else if (!phoneInfo && inputNumber.length > 2)
      phoneInfo = { country: findFirstCountryByCode(inputNumber) };

    const computedMask = getMaskToLibPhoneNumber(phoneInfo);
    const computedCountry = phoneInfo ? phoneInfo.country : '';

    if (visitorCountry) {
      if (!inputNumber) inputNumber = computedMask.countryCode;
      this.setState({ visitorCountry: null });
    }

    if (computedMask && computedMask.mask)
      this.setState({
        mask: computedMask.mask,
        country: computedCountry,
        countryCode: computedMask.countryCode,
      });
    else if (countryCode.length > inputNumber.length)
      this.setState({
        mask: defaultMask,
        country: '',
        countryCode: '',
      });

    this.setState({ inputValue: inputNumber });
  };

  render() {
    const { error, touched, placeholder, onChange } = this.props;
    const { inputValue, mask, country } = this.state;

    return (
      <View>
        <View style={styles.inputWrapper}>
          <TextInputMask
            type={'custom'}
            ref={ref => (this.phoneField = ref)}
            options={{
              mask: mask,
            }}
            value={inputValue}
            placeholder={placeholder}
            onChangeText={number => {
              this.updateMaskData(number);
              if (onChange) onChange(number);
            }}
            style={styles.input}
            placeholderTextColor="rgba(0,0,0,0.7)"
            underlineColorAndroid="rgba(0,0,0,0)"
            keyboardType="phone-pad"
          />
          <View style={styles.flagWrapper}>
            {country ? <Flag code={country} size={32} style={styles.flag} /> : null}
          </View>
        </View>
        <View style={error && touched ? styles.lineError : styles.line} />
        <Text style={styles.error}>{error && touched ? error : null}</Text>
      </View>
    );
  }
}

PhoneMaskInput.propTypes = {
  value: PropTypes.string,
  placeholder: PropTypes.string,
  onChange: PropTypes.func,
};

PhoneMaskInput.defaultProps = {
  value: '',
  placeholder: '',
};

const styles = StyleSheet.create({
  inputWrapper: {
    flex: 1,
    width: 250,
    flexDirection: 'row',
  },
  flagWrapper: {
    flex: 1,
    height: 40,
    backgroundColor: '#fff',
    paddingRight: 5,
    justifyContent: 'center',
    alignItems: 'center',
  },
  input: {
    flex: 7,
    height: 40,
    alignSelf: 'stretch',
    fontSize: 20,
    paddingLeft: 10,
    color: 'rgba(0,0,0,1)',
    backgroundColor: '#fff',
  },
  line: {
    width: 250,
    borderWidth: 0.5,
    borderColor: 'rgba(255,255,255,0.5)',
    marginVertical: 10,
  },
  lineError: {
    width: 250,
    borderWidth: 0.5,
    borderColor: 'rgba(255,0,0,1)',
    marginVertical: 10,
  },
  error: {
    height: 20,
    color: 'rgba(255,0,0,1)',
  },
});

export default PhoneMaskInput;
